use actix_web::{web, HttpResponse, Responder};
use chrono::{Days, NaiveDate, NaiveTime};
use serde::Deserialize;
use sqlx::PgPool;

use crate::{
    endpoints::availability::NewOrEditAvailabilityTemplate,
    utils::{ApplicationError, TemplateResponse},
};
use brass_db::models::{find_free_date_time_slots, Availability, User};

#[derive(Deserialize)]
struct AvailabilityNewQuery {
    date: NaiveDate,
}

#[actix_web::get("/availability/new")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    query: web::Query<AvailabilityNewQuery>,
) -> Result<impl Responder, ApplicationError> {
    let availabilities_from_user =
        Availability::read_by_user_and_date(pool.get_ref(), user.id, &query.date).await?;
    let slot_suggestions = find_free_date_time_slots(&availabilities_from_user);

    let user_can_create_availability =
        availabilities_from_user.is_empty() || !slot_suggestions.is_empty();

    if !user_can_create_availability {
        return Ok(HttpResponse::BadRequest().finish());
    }

    let template = NewOrEditAvailabilityTemplate {
        user: user.into_inner(),
        date: query.date,
        enddate: None,
        id: None,
        start: Some(NaiveTime::from_hms_opt(10, 0, 0).unwrap()),
        end: Some(NaiveTime::from_hms_opt(20, 0, 0).unwrap()),
        comment: None,
        slot_suggestions,
        datetomorrow: query.date.checked_add_days(Days::new(1)).unwrap(),
    };

    Ok(template.to_response()?)
}
